@@ -17,14 +17,13 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
 #include "exec.h"
 #include "gdbarch.h"
 #include "inferior.h"
 #include "target.h"
 #include "command.h"
 #include "completer.h"
-#include "gdbcmd.h"
+#include "cli/cli-cmds.h"
 #include "gdbthread.h"
 #include "ui-out.h"
 #include "observable.h"
@@ -85,8 +84,6 @@ inferior::~inferior ()
      that target, leaving its reference count artificially high.  However,
      this is not critical as the dummy_target is a singleton.  */
   gdb_assert (m_target_stack.top ()->stratum () == dummy_stratum);
-
-  m_continuations.clear ();
 }
 
 inferior::inferior (int pid_, bool hidden)
@@ -181,20 +178,15 @@ inferior::set_args (gdb::array_view<char * const> args)
 }
 
 void
-inferior::add_continuation (std::function<void ()> &&cont)
+inferior::set_arch (gdbarch *arch)
 {
-  m_continuations.emplace_front (std::move (cont));
-}
+  gdb_assert (arch != nullptr);
+  gdb_assert (gdbarch_initialized_p (arch));
+  m_gdbarch = arch;
 
-void
-inferior::do_all_continuations ()
-{
-  while (!m_continuations.empty ())
-    {
-      auto iter = m_continuations.begin ();
-      (*iter) ();
-      m_continuations.erase (iter);
-    }
+  process_stratum_target *proc_target = this->process_target ();
+  if (proc_target != nullptr)
+    registers_changed_ptid (proc_target, ptid_t (this->pid));
 }
 
 /* Notify interpreters and observers that inferior INF was added.  */
@@ -523,24 +515,22 @@ number_of_inferiors (void)
    special cases the null process and devices.  */
 
 const std::string
-inferior_pid_to_str (const inferior *inf)
+inferior_pid_to_str (inferior *inf)
 {
   const int pid = inf->pid;
   if (pid != 0)
     {
-      gdbarch *target_gdbarch = inf->gdbarch;
+      gdbarch *target_gdbarch = inf->arch ();
       if ((target_gdbarch != nullptr)
 	  && gdbarch_is_inferior_device (target_gdbarch))
 	{
 	  const target_desc *tdesc = gdbarch_target_desc (target_gdbarch);
-	  const std::string pci_slot
-	    = tdesc_find_device_info_attribute (tdesc, "pci_slot");
-	  const std::string subdevice_id
-	    = tdesc_find_device_info_attribute (tdesc, "subdevice_id");
+	  const tdesc_device *device_info = tdesc_device_info (tdesc);
+	  const std::string pci_slot = device_info->pci_slot;
 
-	  if (subdevice_id != "")
-	    return string_printf ("device [%s].%s", pci_slot.c_str (),
-				  subdevice_id.c_str ());
+	  if (device_info->subdevice_id.has_value ())
+	    return string_printf ("device [%s].%d", pci_slot.c_str (),
+				  *(device_info->subdevice_id));
 	  else
 	    return string_printf ("device [%s]", pci_slot.c_str ());
 	}
@@ -637,7 +627,7 @@ print_inferior (struct ui_out *uiout, const char *requested_inferiors,
       /* Check if the inferior is a device to display the additional help
 	 later.  */
       device_in_inferiors = (device_in_inferiors
-			     || gdbarch_is_inferior_device (inf->gdbarch));
+			     || gdbarch_is_inferior_device (inf->arch ()));
 
       inferiors_to_print.push_back (inf);
     }
@@ -793,8 +783,6 @@ kill_inferior_command (const char *args, int from_tty)
 
       target_kill ();
     }
-
-  bfd_cache_close_all ();
 }
 
 /* See inferior.h.  */
@@ -809,10 +797,10 @@ switch_to_inferior_no_thread (inferior *inf)
 
 /* See regcache.h.  */
 
-gdb::optional<scoped_restore_current_thread>
+std::optional<scoped_restore_current_thread>
 maybe_switch_inferior (inferior *inf)
 {
-  gdb::optional<scoped_restore_current_thread> maybe_restore_thread;
+  std::optional<scoped_restore_current_thread> maybe_restore_thread;
   if (inf != current_inferior ())
     {
       maybe_restore_thread.emplace ();
@@ -957,15 +945,13 @@ remove_inferior_command (const char *args, int from_tty)
 struct inferior *
 add_inferior_with_spaces (bool hidden)
 {
-  struct address_space *aspace;
   struct program_space *pspace;
   struct inferior *inf;
 
   /* If all inferiors share an address space on this system, this
      doesn't really return a new address space; otherwise, it
      really does.  */
-  aspace = maybe_new_address_space ();
-  pspace = new program_space (aspace);
+  pspace = new program_space (maybe_new_address_space ());
   inf = add_inferior (0, hidden);
   inf->pspace = pspace;
   inf->aspace = pspace->aspace;
@@ -973,10 +959,10 @@ add_inferior_with_spaces (bool hidden)
   /* Setup the inferior's initial arch, based on information obtained
      from the global "set ..." options.  */
   gdbarch_info info;
-  inf->gdbarch = gdbarch_find_by_info (info);
+  inf->set_arch (gdbarch_find_by_info (info));
   /* The "set ..." options reject invalid settings, so we should
      always have a valid arch by now.  */
-  gdb_assert (inf->gdbarch != NULL);
+  gdb_assert (inf->arch () != nullptr);
 
   return inf;
 }
@@ -1131,19 +1117,17 @@ clone_inferior_command (const char *args, int from_tty)
 
   for (i = 0; i < copies; ++i)
     {
-      struct address_space *aspace;
       struct program_space *pspace;
       struct inferior *inf;
 
       /* If all inferiors share an address space on this system, this
 	 doesn't really return a new address space; otherwise, it
 	 really does.  */
-      aspace = maybe_new_address_space ();
-      pspace = new program_space (aspace);
+      pspace = new program_space (maybe_new_address_space ());
       inf = add_inferior (0);
       inf->pspace = pspace;
       inf->aspace = pspace->aspace;
-      inf->gdbarch = orginf->gdbarch;
+      inf->set_arch (orginf->arch ());
 
       switch_to_inferior_and_push_target (inf, no_connection, orginf);
 
@@ -1202,7 +1186,6 @@ static const struct internalvar_funcs inferior_funcs =
   NULL,
 };
 
-
 /* Helper for info devices command.  Prints all currently active devices,
    meaning all devices currently associated with an inferior.  */
 
@@ -1210,32 +1193,22 @@ static void
 print_devices (ui_out *uiout)
 {
   /* Collect all known device inferiors (including hidden inferiors).  The
-     devices are displayed lexigraphically ordered by PCI slot, subdevice_id,
-     vendor_id and target_id.  */
-  std::map<std::tuple<std::string, std::string, std::string, std::string,
-		      std::string>,
-	   std::vector<const inferior *>> devices_to_print;
-  for (const inferior *inf : all_inferiors ())
+     devices are displayed lexigraphically ordered by connection_id and
+     device_uuid.  */
+  std::map<std::tuple<int, std::string>, inferior *> devices_to_print;
+  for (inferior *inf : all_inferiors ())
     {
       /* Only display devices connected to an inferior.  */
-      if (!gdbarch_is_inferior_device (inf->gdbarch))
+      if (!gdbarch_is_inferior_device (inf->arch ()))
 	continue;
 
-      const target_desc *tdesc = gdbarch_target_desc (inf->gdbarch);
-      const std::string pci_slot
-	= tdesc_find_device_info_attribute (tdesc, "pci_slot");
-      const std::string subdev_id
-	= tdesc_find_device_info_attribute (tdesc, "subdevice_id");
-      const std::string vendor_id
-	= tdesc_find_device_info_attribute (tdesc, "vendor_id");
-      const std::string target_id
-	= tdesc_find_device_info_attribute (tdesc, "target_id");
-      const std::string device_uuid
-	= tdesc_find_device_info_attribute (tdesc, "device_uuid");
+      const target_desc *tdesc = gdbarch_target_desc (inf->arch ());
+      std::string device_uuid = tdesc_device_info (tdesc)->uuid;
+      int connection_id
+	= inf->process_target ()->connection_number;
 
-      devices_to_print[std::make_tuple (pci_slot, subdev_id, vendor_id,
-					target_id, device_uuid)]
-	.push_back (inf);
+      devices_to_print[std::make_tuple (connection_id, device_uuid)]
+	= inf;
     }
 
   if (devices_to_print.empty ())
@@ -1250,14 +1223,15 @@ print_devices (ui_out *uiout)
   int current_device = -1;
 
   {
-    gdb::optional<ui_out_emit_list> list_emitter;
-    gdb::optional<ui_out_emit_table> table_emitter;
+    std::optional<ui_out_emit_list> list_emitter;
+    std::optional<ui_out_emit_table> table_emitter;
 
     if (!uiout->is_mi_like_p ())
       {
-	table_emitter.emplace (uiout, 7, devices_to_print.size (),
+	table_emitter.emplace (uiout, 8, devices_to_print.size (),
 			       "InfoDevicesTable");
 	uiout->table_header (1, ui_left, "current", "");
+	uiout->table_header (5, ui_left, "number", "Num");
 	uiout->table_header (15, ui_left, "location", "Location");
 	uiout->table_header (12, ui_left, "sub-device", "Sub-device");
 	uiout->table_header (11, ui_left, "vendor-id", "Vendor Id");
@@ -1274,23 +1248,13 @@ print_devices (ui_out *uiout)
 
     for (const auto& device_id_desc_pair : devices_to_print)
       {
-	const std::vector<const inferior *> &inf = device_id_desc_pair.second;
-	/* gdbarch is the same for all inferiors.  */
-	const target_desc *tdesc = gdbarch_target_desc (inf[0]->gdbarch);
-
-	const std::string &pci_slot = std::get<0> (device_id_desc_pair.first);
-	const std::string &subdev_id = std::get<1> (device_id_desc_pair.first);
-	const std::string &vendor_id = std::get<2> (device_id_desc_pair.first);
-	const std::string &target_id = std::get<3> (device_id_desc_pair.first);
-	const std::string total_cores
-	  = tdesc_find_device_info_attribute (tdesc, "total_cores");
-	const std::string device_name
-	  = tdesc_find_device_info_attribute (tdesc, "device_name");
+	inferior *inf = device_id_desc_pair.second;
+	const target_desc *tdesc = gdbarch_target_desc (inf->arch ());
+	const tdesc_device *dev_info = tdesc_device_info (tdesc);
 
 	ui_out_emit_tuple tuple_emitter (uiout, nullptr);
 
-	if (std::find (inf.begin (), inf.end (), current_inferior ())
-	    != inf.end ())
+	if (inf == current_inferior ())
 	  {
 	    current_device = counter;
 	    if (!uiout->is_mi_like_p ())
@@ -1299,35 +1263,42 @@ print_devices (ui_out *uiout)
 	else if (!uiout->is_mi_like_p ())
 	  uiout->field_skip ("current");
 
-	if (uiout->is_mi_like_p ())
-	  uiout->field_unsigned ("number", counter);
+	uiout->field_unsigned ("number", counter);
 
 	uiout->field_string ("location",
 			     (uiout->is_mi_like_p ()
-			      ? string_printf ("%s", pci_slot.c_str ())
-			      : string_printf ("[%s]", pci_slot.c_str ())));
+			      ? string_printf ("%s",
+					       dev_info->pci_slot.c_str ())
+			      : string_printf ("[%s]",
+					       dev_info->pci_slot.c_str ())));
 
-	if (subdev_id.empty ())
-	  uiout->field_string ("sub-device", "-");
+	if (dev_info->subdevice_id.has_value ())
+	  uiout->field_unsigned ("sub-device", *(dev_info->subdevice_id));
 	else
-	  uiout->field_string ("sub-device", subdev_id.c_str ());
+	  uiout->field_string ("sub-device", "-");
 
-	uiout->field_string ("vendor-id", vendor_id.c_str ());
+	if (dev_info->vendor_id.has_value ())
+	  uiout->field_fmt ("vendor-id", "0x%04x", *(dev_info->vendor_id));
+	else
+	  uiout->field_string ("vendor-id", "-");
 
-	uiout->field_string ("target-id", target_id.c_str ());
+	if (dev_info->target_id.has_value ())
+	  uiout->field_fmt ("target-id", "0x%04x", *(dev_info->target_id));
+	else
+	  uiout->field_string ("target-id", "-");
 
-	uiout->field_string ("cores", total_cores.c_str ());
+	if (dev_info->total_cores.has_value ())
+	  uiout->field_unsigned ("cores", *(dev_info->total_cores));
+	else
+	  uiout->field_string ("cores", "?");
 
-	uiout->field_string ("device-name", device_name.c_str ());
+	uiout->field_string ("device-name", dev_info->name.c_str ());
 
 	if (!uiout->is_mi_like_p ())
 	  uiout->text ("\n");
 	else
-	  {
-	    ui_out_emit_list group_list_emitter (uiout, "thread-groups");
-	    for (const inferior *i : inf)
-	      uiout->field_fmt (nullptr, "i%d", i->num);
-	  }
+	  uiout->field_string ("thread-groups",
+			       string_printf ("i%d", inf->num));
 
 	counter++;
       }
@@ -1361,8 +1332,10 @@ mi_cmd_device_info (const char *command, const char *const * argv, int argc)
   print_devices (current_uiout);
 }
 
+/* See inferior.h.  */
+
 void
-initialize_inferiors (void)
+initialize_inferiors ()
 {
   struct cmd_list_element *c = NULL;
 
