@@ -19,7 +19,6 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "gdbsupport/common-defs.h"
 #include "linux-btrace.h"
 #include "gdbsupport/common-regcache.h"
 #include "gdbsupport/gdb_wait.h"
@@ -284,13 +283,12 @@ perf_event_read_bts (btrace_target_info *tinfo, const uint8_t *begin,
   struct perf_event_sample sample;
   size_t read = 0;
   struct btrace_block block = { 0, 0 };
-  struct regcache *regcache;
 
   gdb_assert (begin <= start);
   gdb_assert (start <= end);
 
   /* The first block ends at the current pc.  */
-  regcache = get_thread_regcache_for_ptid (tinfo->ptid);
+  reg_buffer_common *regcache = get_thread_regcache_for_ptid (tinfo->ptid);
   block.end = regcache_read_pc (regcache);
 
   /* The buffer may contain a partial record as its last entry (i.e. when the
@@ -417,58 +415,49 @@ cpu_supports_bts (void)
     }
 }
 
-/* Read config bits.  */
+/* Return the Intel PT config bit from the linux sysfs for a FEATURE.
+   The bit can be used in the perf_event configuration when enabling PT.  */
 
-static bool
-linux_read_pt_config_bit (const std::string &feature, uint64_t *config_bit)
+static uint64_t
+linux_read_pt_config_bit (const char *feature)
 {
-  if (config_bit == nullptr)
-    return false;
-
+  uint64_t config_bit = 0;
   std::string filename
-      = "/sys/bus/event_source/devices/intel_pt/format/" + feature;
-  gdb_file_up file = gdb_fopen_cloexec (filename.c_str (), "r");
+      = std::string ("/sys/bus/event_source/devices/intel_pt/format/")
+      + feature;
 
+  gdb_file_up file = gdb_fopen_cloexec (filename.c_str (), "r");
   if (file.get () == nullptr)
     return false;
 
-  int found = fscanf (file.get (), "config:%" SCNu64, config_bit);
-
+  int found = fscanf (file.get (), "config:%" SCNu64, &config_bit);
   if (found != 1)
-    {
-      warning (_("Failed to determine config bit from %s."),
-	       filename.c_str ());
-      return false;
-    }
+    error (_("Failed to determine config bit from %s."),  filename.c_str ());
 
-  return true;
+  return config_bit;
 }
 
-
-/* Check whether the linux target supports Intel Processor Trace PTWRITE.  */
+/* Check whether the linux target supports the Intel PT FEATURE.  */
 
 static bool
-linux_supports_ptwrite (uint64_t *config_bit)
+linux_supports_pt_feature (const char *feature)
 {
-  static const char filename[]
-      = "/sys/bus/event_source/devices/intel_pt/caps/ptwrite";
-  gdb_file_up file = gdb_fopen_cloexec (filename, "r");
+  std::string filename
+    = std::string ("/sys/bus/event_source/devices/intel_pt/caps/") + feature;
 
+  gdb_file_up file = gdb_fopen_cloexec (filename.c_str (), "r");
   if (file.get () == nullptr)
     return false;
 
   int status, found = fscanf (file.get (), "%d", &status);
-
   if (found != 1)
     {
-      warning (_("Failed to determine ptwrite support from %s."), filename);
+      warning (_("Failed to determine %s support from %s."), feature,
+	       filename.c_str ());
       return false;
     }
 
-  if (status == 1 && linux_read_pt_config_bit ("ptw", config_bit))
-    return true;
-
-  return false;
+  return (status == 1);
 }
 
 /* The perf_event_open syscall failed.  Try to print a helpful error
@@ -477,7 +466,8 @@ linux_supports_ptwrite (uint64_t *config_bit)
 static void
 diagnose_perf_event_open_fail ()
 {
-  switch (errno)
+  int orig_errno = errno;
+  switch (orig_errno)
     {
     case EPERM:
     case EACCES:
@@ -498,7 +488,7 @@ diagnose_perf_event_open_fail ()
       break;
     }
 
-  error (_("Failed to start recording: %s"), safe_strerror (errno));
+  error (_("Failed to start recording: %s"), safe_strerror (orig_errno));
 }
 
 /* Get the linux version of a btrace_target_info.  */
@@ -522,7 +512,7 @@ linux_enable_bts (ptid_t ptid, const struct btrace_config_bts *conf)
     error (_("BTS support has been disabled for the target cpu."));
 
   std::unique_ptr<linux_btrace_target_info> tinfo
-    { gdb::make_unique<linux_btrace_target_info> (ptid) };
+    { std::make_unique<linux_btrace_target_info> (ptid) };
 
   tinfo->conf.format = BTRACE_FORMAT_BTS;
 
@@ -671,7 +661,7 @@ linux_enable_pt (ptid_t ptid, const struct btrace_config_pt *conf)
     pid = ptid.pid ();
 
   std::unique_ptr<linux_btrace_target_info> tinfo
-    { gdb::make_unique<linux_btrace_target_info> (ptid) };
+    { std::make_unique<linux_btrace_target_info> (ptid) };
 
   tinfo->conf.format = BTRACE_FORMAT_PT;
 
@@ -682,13 +672,23 @@ linux_enable_pt (ptid_t ptid, const struct btrace_config_pt *conf)
   tinfo->attr.exclude_hv = 1;
   tinfo->attr.exclude_idle = 1;
 
-  uint64_t config_bit;
-  if (conf->ptwrite && linux_supports_ptwrite (&config_bit))
+  if (conf->ptwrite && linux_supports_pt_feature ("ptwrite"))
     {
-      tinfo->attr.config |= 1 << config_bit;
+      tinfo->attr.config |= 0x1ull << linux_read_pt_config_bit ("ptw");
       tinfo->conf.pt.ptwrite = conf->ptwrite;
     }
 
+  if (conf->event_tracing)
+    {
+      if (linux_supports_pt_feature ("event_trace"))
+	{
+	  tinfo->attr.config |= 0x1ull << linux_read_pt_config_bit ("event");
+	  tinfo->conf.pt.event_tracing = conf->event_tracing;
+	}
+      else
+	error (_("Event tracing for record btrace pt is not supported."));
+    }
+
   errno = 0;
   scoped_fd fd (syscall (SYS_perf_event_open, &tinfo->attr, pid, -1, -1, 0));
   if (fd.get () < 0)
@@ -956,8 +956,7 @@ linux_read_pt (btrace_data_pt *btrace, linux_btrace_target_info *tinfo,
     case BTRACE_READ_NEW:
       if (!perf_event_new_data (&tinfo->pev))
 	return BTRACE_ERR_NONE;
-
-      /* Fall through.  */
+      [[fallthrough]];
     case BTRACE_READ_ALL:
       perf_event_read_all (&tinfo->pev, &btrace->data, &btrace->size);
       return BTRACE_ERR_NONE;
